/*;  port.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.2 $$ {\rhead{\sl port.h\quad\thepage}} */
/*;  $$Date: 2000/09/02 12:46:05 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/port.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) port.h -- Abstract base class for creating ports

#if !defined(port_h)
#define port_h

#include <stdio.h>
#include <time.h>

#include "common.h"     //; ץˤ붦
#include "object.h"     //; 饹 Object 

/*----------------------------------------------------------------------
CLASS
    Port

    Abstract class for creating ports

DESCRIPTION

    Port is the base class from which all port classes are derived.

    <japanese>
    饹 Port ϡƤüҥǥδ쥯饹ǤüҤĤ٤
    Ūʴؿ۴ؿȤƤޤ۴ؿΰϡΥ饹
    ǤϡͿ貾۴ؿˤʤäƤޤ

    </japanese>

----------------------------------------------------------------------*/


//; 饹 Port 

class Port : public Object {

//; 1.  ³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Pure virtual function.  Returns data_pack_type.
  //
  // <japanese>
  // ǡѥåɽʸ֤
  //
  // 貾۴ؿǤ. Port 饹, ǡѥåɽʸ
  // ؤΥݥ󥿤֤褦ɬƲ.
  //
  // 㤨мüҥǥΥ饹, ǡѥå "int" ξ,
  // <pre>
  //   char* Data_pack_type(void) { return "int"; }
  // </pre>
  // Τ褦˼ޤ.
  //
  // ³ˤ, ³üҤȤüҤȤ Data_pack_type פ
  // Ȥǧޤ.
  //
  // <pre>
  // RETURN:
  //   char*      ǡѥåηɽʸؤΥݥ.
  // </pre>
  // </japanese>
  virtual char* Data_pack_type(void) = NULL;

public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Set_sender(void* s);
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Register_receiver(void* r);
        /* <japanese>
          Port::Set_sender(void*) (virtual)           üҤ
          Port::Register_receiver(void*) (virtual)    üҤϿ

        Set_sender , üҤΥǡѥåüҤ s ꤹ
        Ǥ.
        饹Ǥ, void* ˥㥹Ȥ줿üҤΥɥ쥹 s 
        , s ȤǤ YES , Ǥʤä NO
        ֤褦ޤ.

        Register_receiver , r 򤳤üҤΥǡѥåüҤȤ
        ϿؿǤ.
        饹Ǥ, void* ˥㥹Ȥ줿üҤΥɥ쥹 r 
        , r ȤϿǤ YES , ϿǤʤä NO
        ֤褦ޤ.

        Port Ǥ, ü, üҤϿǤʤΤ, 顼å
        ϸ NO ֤ޤ.

        ARGUMENT:
            s     void* ˥㥹Ȥ줿üҤΥɥ쥹.
            r     void* ˥㥹Ȥ줿üҤΥɥ쥹.
        RETURN:
            NO     NO ֤. (: port.cc Ƥ
            Set_sender, Register_receiver Ǥ NO ֤Ȥ̣)
        ACCIDENT:
            r, s  Object* ѿǼƤʤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Switch(void);
        /* <japanese>
          Port::Switch(void) (virtual)    ³ȸν

        饹, ³ȸνԤ褦ޤ.
         YES , Ԥ NO ֤ޤ.

        Port Ǥ, 顼åϸ NO ֤ޤ.

        RETURN:
            NO     NO ֤. (: port.cc Ƥ
            Switch Ǥ NO ֤Ȥ̣)
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ׻γǧ
  //
  // ³ȸ, ׻γǧѤޤ.
  //
  // ³Ȥλ, ׻Ȥ˿ʤळȤǤ YES ֤, ³
  // ȤλƤʤ NO ֤ޤ.
  //
  // <pre>
  // RETURN:
  //   YES   ³ȤλƤ.
  //    NO    ³ȤλƤʤ.
  // </pre>
  // </japanese>
  virtual Boolean Are_you_ready(void) = NULL;

  //////////////////////////////////////////////////////////////////////
  // Request of calculation. Transfer the request to a element which
  // caluculate.
  virtual void Creq(void) {};

//;  2.  ǡѥåμǽ, õǽ.
public:
  //////////////////////////////////////////////////////////////////////
  virtual void Set_data_pack_array(void) = NULL;
        /* <japanese>
          Port::Set_data_pack_array(void) (pure virtual)
                ǡݥѥåκ

        饹ˤ, üҤǼǡѥå
        褦ޤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ǽǡѥåγǧ
  //
  // Can_you_get_data(time_t,time_t) , t_from (sec) , t_to
  // (sec) ʹߤޤǤΥǡѥå򤳤üҤǽʤ, YES 
  // ֤, Բǽʤ NO ֤褦, 饹ޤ.
  //
  // <pre>
  //    ARGUMENT:
  //        t_from    t_from ΥǡѥåǽǤ뤳
  //                    ǧ.
  //        t_to      t_to ʹߤޤǤΥǡѥåǽǤ뤳Ȥ
  //                    ǧ.
  //    RETURN:
  //        YES       t_from  t_to ʹߤޤǤΥǡѥå
  //                    ǽʾ.
  //                    t ˤǡǽʾ.
  //        NO        t_from  t_to ʹߤޤǤΥǡѥå
  //                    Բǽʾ.
  // </pre>
  // </japanese>
  virtual Boolean Can_you_get_data(time_t t_from, time_t t_to) = NULL;


  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // Can_you_get_data(time_t) , t ˤǡǽǧ
  // Ǥ.
  // </japanese>
  Boolean Can_you_get_data(time_t t) { return Can_you_get_data(t, t); }

public:
  //////////////////////////////////////////////////////////////////////
  virtual void Get_data_pack_array(void**& dp_array, int& num_of_dp)
                    = NULL;
        /* <japanese>
          Port::Get_data_pack_array(void**&,int&) (pure virtual)
                ǡѥåμ

        饹ˤ, ǡѥåؿޤ.
        dp_array ˤϥǡѥåؤΥݥ󥿤Ǽ, num_of_dp ˤ,
        dp_array ǿǼޤ.

        ARGUMENT:
            dp_array      ǡѥåǼ뻲Ȱ.
            num_of_dp     dp_array ǿǼ뻲Ȱ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  virtual time_t Oldest_data_time(void) = NULL;
  //////////////////////////////////////////////////////////////////////
  virtual time_t Newest_data_time(void) = NULL;
  //////////////////////////////////////////////////////////////////////
  virtual int Number_of_data_packs(void) = NULL;

  //////////////////////////////////////////////////////////////////////
  virtual Boolean Get_data_pack(
                        void*& former, time_t data_time, void*& later,
                        char flag) = NULL;
  //////////////////////////////////////////////////////////////////////
  Boolean Get_data_pack(void*& former, time_t data_time)
        { void* dummy;
          return Get_data_pack(former, data_time, dummy, 'f'); }
  //////////////////////////////////////////////////////////////////////
  Boolean Get_data_pack(time_t data_time, void*& later)
        { void* dummy;
          return Get_data_pack(dummy, data_time, later, 'l'); }
        /* <japanese>
          Port::Get_data_pack(void*&,time_t,void*&,char) (pure virtual)
          Port::Get_data_pack(void*&,time_t) (inline)
          Port::Get_data_pack(time_t,void*&) (inline)
                ǡѥåμ

        饹ˤ, ꤷƥǡѥåؿ
        ޤ.

        former ˤ, ǡϿ郎 data_time  data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        later ˤ, ǡϿ郎 data_time ʹߤ data_time ˺Ǥ
        ϿǡѥåؤΥݥ󥿤, Ǽޤ.

        ǡϿ郎 data_time ǤǡѥåϿƤ
        , former, later Ȥ⤽Ǽޤ.

        ǡѥåǤʤ, former, later  NULL
        ꤵޤ.

        flag ϼΥ⡼ɤ򤹤뤿Υץ, ̤Ǥ.
            'b'       former, later ξɬ.
                        Ǥʤ, ξȤ NULL ꤹ.
            'f'       later ǤʤƤ, former Τ߼.
                        later Ǥʤ, NULL ꤹ.
            'l'       former ǤʤƤ, later Τ߼.
                        former Ǥʤ, NULL ꤹ.
            ¾    former, later ΤʤȤ.
                        ǤʤΤ NULL ꤹ.

        flag Υǥեͤ, 'b' Ǥ.
        ޤ,  later Ϥʤ, flag  f Ȥ former Τ
        ,  former Ϥʤ, flag  l Ȥ later 
        ߼ޤ.

        ARGUMENT:
            former        d_time  d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            data_time     . (sec)
                            λ˺ǤᤤǡϿļϿ
                            ѥå.
            later         d_time ʹߤ d_time ˺ǤᤤϿǡѥ
                            ؤΥݥ󥿤Ǽ뻲Ȱ.
            flag          ǡѥåμꤹե饰.
        RETURN:
            YES           ΥǡѥåǤ.
            NO            data_pack_array  NULL ξ.
                            ΥǡѥåǤʤ.
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_from(void) = NULL;
  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_to(void) = NULL;
        /* <japanese>
          Port::Necessary_time_from(void) (pure virtual)
          Port::Necessary_time_to(void) (pure virtual)
                ǡɬ׻֤

        פʥǡѥåλ뤿ޤ.

        üҤǼǽʥǡѥåΤ, Necessary_time_from 
        ǡϿ郎ᤤǡѥå, Port::Reduce_data_pack_stock
        ˤõޤ.

        RETURN:
            time_t    ǡɬ׻ (sec).
        </japanese> */
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ץǡѥåξõ
  //
  // üҤǼǽʥǡѥåΤ, Necessary_time_from 
  // ǡϿ郎ᤤǡѥåõ褦, 饹
  // ޤ.
  //
  // </japanese>
  virtual void Reduce_data_pack_stock(void) = NULL;

//  3.  ֥Ⱦϵǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // <japanese>
  // ֥Ⱦν
  //
  // Port::Cprint Ǥ, 饹̾, ǡݥѥåη̾, ü̾, ü
  // ֹե fp_out ˲̤Ϥޤ.
  //
  // <pre>
  //   fprintf(fp_out,
  //     "%s%s<%s> %s %d \n",
  //     head,
  //     class_name, Data_pack_type(), object_name, object_number);
  // </pre>
  //
  // ؿ Cprint ϴ쥯饹 Object β۴ؿʤΤ, 饹
  // ѹǤޤ.
  //
  // Object::Cprint ¿Ƥ뤿,  fp_out ά
  //  fp_out Υǥեͤ stderr Ǥ.  fp_out, head ξ
  // Ԥά, ǥեͤϤ줾 stderr, Ǥ.
  //
  // fp_out, head άȤ, stderr, ꤵ褦, ¿
  // ޤ.
  //
  // <pre>
  //    ARGUMENT:
  //     fp_out    ֥ȾϤեؤΥݥ.
  //                 ǥե stderr.
  //     head      ʸؤΥݥ.
  //                 ǥեͤϣ.
  // </pre>
  // </japanese>
  void Cprint(FILE* fp_out, char* head);

public:
  //////////////////////////////////////////////////////////////////////
  virtual void Print_connection(FILE* fp_out, char* head);
  //////////////////////////////////////////////////////////////////////
  void Print_connection(FILE*)
        { Print_connection(stderr, "    "); }
  //////////////////////////////////////////////////////////////////////
  void Print_connection(char* head = "    ")
        { Print_connection(stderr, head); }
        /* <japanese>
          Port::Print_connection(FILE*,char*) (virtual)
          Port::Print_connection(FILE*) (inline)
          Port::Print_connection(char*) (inline)
                ³ν

        饹, ³ɽ褦ޤ.
        Port Ǥ, 顼åϤޤ.

        fp_out, head Υǥեͤ, stderr, Ǥ.
        ޤ.

        ARGUMENT:
            fp_out    եؤΥݥ. ǥե stderr.
            head      ϻΥإå. ǥեͣ.
        </japanese> */

//;  4.  󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns a new pointer to a new Port
  virtual Port* NewPort() = 0;

  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
        { return Object::Init(p_name, p_num); }
        /* <japanese>
          Port::Init(char*,int) (inline)  Ѵؿ

        Object::Init ¹Ԥޤ.

        RETURN:
            YES   .
            NO    Ť˽줿. (Object::Init)
        EXIT:
            5     object_name εΰ褬ݤǤʤ.
                    (Object::Init)
        </japanese> */
  //////////////////////////////////////////////////////////////////////
  Port(char* p_name, int p_num);
  //////////////////////////////////////////////////////////////////////
  Port(void);
  //////////////////////////////////////////////////////////////////////
  ~Port(void);
        /* <japanese>
          Port::Port(char*,int)           󥹥ȥ饯
          Port::Port(void)                󥹥ȥ饯
          Port::~Port(void) (virtual)     ǥȥ饯

        Port(char*,int) , 쥯饹 Object Υǥեȥ
        ȥ饯Ѥ, Port::Init Ѥ, Фʲ̤
        ޤ.

            class_name        : "Port"
            object_name       : p_name (ҡפ˥ԡ)
            object_number     : p_num
            owner             : NULL

        ǥեȥ󥹥ȥ饯, Port::Init ¹Ԥޤ.
        η, ʲȤʤޤ.
            object_name       : NULL
            object_number     : -1
        ¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
        ξ, Object::Init ѤƥФƲ.

        ǥȥ饯, ܴ쥯饹 Object  virtual 
        Ƥޤ.

        ARGUMENT:
            p_name    ü̾.
            p_num     üֹ.
        EXIT:
            5         object_name εΰݤǤʤ.
                        (Object::Init)
        </japanese> */

};


#endif  //  ! port_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
