/* sender.h {\documentstyle{jsrc2latex}\rhead{\sl sender.h\quad\thepage}
   \font\tenrm=cmtt10}ܷüҥǥκ
{\ \begin{verbatim}
$Revision: 1.1 $ $Date: 1994/12/17 12:31:36 $  by KyotoHydro
\end{verbatim}}
{\ {\setlength{\parskip}{4pt}% latex-mode

ܷüҥǥ \verb|Sender_port| ޤ
}% end of latex-mode }
*/

#if !defined(sender_h)
#define sender_h

#include <stdio.h>
#include <time.h>		//; time_t , tm 

#include "common.h"		//; ץˤ붦
#include "list.h"		//; 饹 List 
#include "port.h"		//; 饹 Port 


//;	饹 Sender_port .

/* ---------------------------------------------------------------------
CLASS
  Sender_port

  Abstract class of Sender ports

DESCRIPTION
  Abstract class for creating sender ports.

  By inheritance from this Sender_port class, the classes Send_port and
  Relay_port are created.

  <japanese>
  sender port 뤿ݥ饹Ǥsender port ϡǽ
  äüҤǤ

  饹 Send_port  Relay_port Υ饹ˤäƺ
  ޤ

  <ul>
   <li> Send_port ϡʬǡäƤü
   <li> Relay_port ϡ
        ʬǤϥǡäƤʤüҤ׵Ĥ
        ¾üҤǡäžüҤǤ
  </ul>

  ˤƤ⡢¦餹СǡƤ뵡ǽäƤ
  ΤǡζʬΥ󥿡ե Sender_port Ƥޤ

  Sender_port ˤʣü (ǡü) ³뤳
  Ǥޤ
  </japanese>

--------------------------------------------------------------------- */

class Sender_port : public Port {
	/* <japanese>
		Sender_port		ܷüҥǥΥ饹

	饹 Sender_port Ǥ,
		ǡѥåü	: receiver_array, number_of_receivers
	򵭲,
		üҤ³ǽ
		ǡѥåǽ
	, ޤ.

	Sender_port ˤʣüҤ³Ǥޤ.
	*/

//;	0.	ǡ.
protected:
  //////////////////////////////////////////////////////////////////////
  // List used internal work
  List*	buffer_list;			//; ѥꥹ

  //////////////////////////////////////////////////////////////////////
  // Array of ports which receive data from this port
  void**	receiver_array;			//; üҵ

  //////////////////////////////////////////////////////////////////////
  // number of ports which receive data from this port
  int		number_of_receivers;	//; receiver_array ǿ

  //////////////////////////////////////////////////////////////////////
  // Comment about data
  char*   comment_string;         //; comment about data

//;	1.	³ǽ.
public:
  //////////////////////////////////////////////////////////////////////
  // Returns the datapack-type string of this port. Because this is a
  // pure virtual function, it should be implemented in derived classes.
  //
  virtual char* Data_pack_type(void) = NULL;
		/* <japanese>
			Send_port::Data_pack_type(void) (pure virtual)
				ǡѥåɽʸ֤

		쥯饹 Port ν貾۴ؿǤ.
		Sender_port Ǥ貾۴ؿȤƤޤ.
		Sender_port 饹, ǡѥåɽʸؤΥ
		󥿤֤褦ɬƲ.

		㤨, ǡѥå "INT" Ǥ饹Ǥ,
			char* Data_pack_type(void) { return "INT"; }
		Τ褦ޤ.

		³ˤ, üҤüҤȤ Data_pack_type פ뤳
		Ȥǧޤ.

		RETURN:
			char*		ǡѥåɽʸؤΥݥ.
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  // Register a receiver.  This function is a pure virtual function in
  // the base class Port.  Implemented in this class Sener_port.
  //
  virtual Boolean Register_receiver(void* r);
		/* <japanese>
			Sender_port::Register_receiver(void*) (virtual)
				üҤϿ

		쥯饹 Port ν貾۴ؿǤ.
		List::Put_tail ˤ buffer_list  r Ͽ, YES ֤ޤ.

		, ξϥ顼åɽ NO ֤ޤ.
			buffer_list  NULL ξ.
			receive_port_array Ѥߤξ.
			üҤ s , Data_pack_type ۤʤ.

		ޤ, r  buffer_list ϿѤߤξ, Ͽ YES 
		ޤ.

		ARGUMENT:
			r		void* ˥㥹Ȥ줿üҤΥɥ쥹.
		RETURN:
			YES	Ͽ줿.
			NO	buffer_list  NULL ξ.
					receive_port_array Ѥߤξ.
					üҤ s , Data_pack_type ۤʤ.
		EXIT:
			5		buffer_list ΤεΰݤǤʤ.
					(List::Put_tail)
		ACCIDENT:
			r  Port* ѿǼƤʤ.
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Switch(void);
		/* <japanese>
			Sender_port::Switch(void) (virtual)		buffer_list ž

		쥯饹 Port ν貾۴ؿǤ.
		buffer_list ϿƤüҤΥɥ쥹 receiver_array
		λؤ󲽤, receiver_array ǿ number_of_receivers
		ꤷޤ.

		receiver_array Ѥߤξ, 顼åɽ NO
		֤ޤ.

		³Ȥνλ, owner η׻ɬѤƲ. ,
		δؿμ¹Ը, ³طѹǤޤ.

		RETURN:
			YES	receiver_array ˺줿.
			NO	receiver_array Ѥߤξ.
		EXIT:
			5		receiver_array εΰݤǤʤä.
					(List::List_to_array)
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Are_you_ready(void)
		{ return (number_of_receivers >= 0) ? YES : NO; }
		/* <japanese>
			Sender_port::Are_you_ready(void) (virtual, inline)
				׻γǧ

		쥯饹 Port ν貾۴ؿǤ.

		ǡѥåǽʾ֤Ǥ뤳Ȥǧޤ.
		number_of_receivers  0 ʾʤ, receiver_array Ѥߤ,
		üҤ˥ǡѥåǤޤ.

		RETURN:
			YES	receiver_array Ѥߤξ.
			NO	receiver_array ̤ξ.
        </japanese>
		*/

public:
  //////////////////////////////////////////////////////////////////////
  // Copy the string <b>cmt</b> to <b>comment_string</b> which is a data
  // member of this class.
  void Set_comment_string(char* cmt);

  //////////////////////////////////////////////////////////////////////
  // Creates a string and copy <b>comment_string</b> to the string.
  // This function returns the string.
  char* Get_comment_string(void);

//;	2.	ǡѥåǽ, õǽ.
public:
  //////////////////////////////////////////////////////////////////////
  virtual void Set_data_pack_array(void) = NULL;

  //////////////////////////////////////////////////////////////////////
  virtual Boolean Can_you_get_data(time_t t_from, time_t t_to) = NULL;

  //////////////////////////////////////////////////////////////////////
  virtual void Get_data_pack_array(void**& dp_array, int& num_of_dp)
						= NULL;
  //////////////////////////////////////////////////////////////////////
  virtual Boolean Get_data_pack(
						void*& former, time_t data_time, void*& later,
						char flag) = NULL;

  //////////////////////////////////////////////////////////////////////
  virtual void Reduce_data_pack_stock(void) = NULL;
		/* <japanese>
			Port::Set_data_pack_array(void) (pure virtual)
				ǡݥѥåκ
			Sender_port::Can_you_get_data(time_t,time_t) (pure virtual)
				ǽǡѥåγǧ
			Sender_port::Get_data_pack_array(void**&,int&) (pure virtual)
				ǡѥå
			Sender_port::Get_data_pack(void*&,time_t,void*&,char)
				(pure virtual)	ǡѥå
			Sender_port::Reduce_data_pack_stock(void) (pure virtual)
				ץǡѥåξõ

		嵭δؿ, 쥯饹 Port ν貾۴ؿǤ.
		Sender_port Ǥϼ, 饹Ǽޤ.
        </japanese>
		*/
public:
  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_from(void);

  //////////////////////////////////////////////////////////////////////
  virtual time_t Necessary_time_to(void);
		/* <japanese>
			Sender_port::Necessary_time_from(void)
			Sender_port::Necessary_time_to(void)
				ǡɬ׻֤

		쥯饹 Port ν貾۴ؿǤ.

		Necessary_time_from Ǥ, owner  Element::Necessary_time_from,
		üҤ Port::Necessary_time_from Τ, ǾΤΤ֤
		. λ, owner  NULL ʤ, LONG_MIN ֤ޤ.

		Necessary_time_to Ǥ, owner  Element::Necessary_time_to, 
		üҤ Port::Necessary_time_to Τ, ΤΤ֤ޤ. 
		λ, owner  NULL ʤ, LONG_MAX ֤ޤ.

		RETURN:
			time_t	ǡɬ׻ (sec).
        </japanese>
		*/

//;	3.	ϵǽ.
public:
  //////////////////////////////////////////////////////////////////////
  void Print_connection(FILE* fp_out, char* head);
		/* <japanese>
			Sender_port::Print_connection(FILE*,char*) (virtual)
				³ν

		쥯饹 Port β۴ؿǤ.
		üҤ³ط˴ؤ fp_out ˽Ϥޤ.

		ARGUMENT:
			fp_out	եؤΥݥ. ǥե stderr.
			head		ϻΥإå. ǥեͣ.
        </japanese>
		*/

//;	4.	󥹥ȥ饯, ǥȥ饯.
public:
  //////////////////////////////////////////////////////////////////////
  Boolean Init(char* p_name, int p_num)
		{ return Port::Init(p_name, p_num); }
		/* <japanese>
			Sender_port::Init(char*,int) (inline)	Ѵؿ

		Port::Init ¹Ԥޤ.

		RETURN:
			YES	.
			NO	Ť˽줿. (Object::Init)
		EXIT:
			5		object_name εΰ褬ݤǤʤ.
					(Object::Init)
        </japanese>
		*/
  //////////////////////////////////////////////////////////////////////
  Sender_port(char* p_name, int p_num);

  //////////////////////////////////////////////////////////////////////
  Sender_port(void);

  //////////////////////////////////////////////////////////////////////
  ~Sender_port(void);
		/* <japanese>
			Sender_port::Sender_port(char*,int)			󥹥ȥ饯
			Sender_port::Sender_port(void)				󥹥ȥ饯
			Sender_port::~Sender_port(void) (virtual)	ǥȥ饯

		Sender_port(char*,int) , 쥯饹 Port Υǥեȥ󥹥ȥ
		Ѥ, Sender_port::Init Ѥ, Фʲ̤
		ޤ.
			class_name			: "Sender_port"
			object_name			: p_name (ҡפ˥ԡ)
			object_number			: p_num
			owner					: NULL
			buffer_list			: ҡפ˳ݤ. (³)
			receiver_array		: NULL
			number_of_receivers	: -1

		ǥեȥ󥹥ȥ饯, Sender_port::Init ¹Ԥޤ.
		η, ʲȤʤޤ.
			object_name			: NULL
			object_number			: -1
		¾ΥФФ, Ĥ󥹥ȥ饯ƱǤ.
		ξ, Object::Init ѤƥФƲ.

		ǥȥ饯, ܴ쥯饹 Object  virtual Ƥޤ.
		Sender_port Ǥ, buffer_list, receiver_array  delete ޤ.

		ARGUMENT:
			p_name	ü̾.
			p_num		üֹ.
		EXIT:
			5			object_name εΰݤǤʤ.
						(Object::Init)
			5			buffer_list εΰݤǤʤ. (new)
        </japanese>
		*/

};

#endif	//	! sender_h

/* end of file {\ % The following part will be omitted in tex file
\begin{comment}\begin{verbatim}
This part is not included in the tex output.
$Revision: 1.1 $

# Local Variables: ***
# tab-width: 4 ***
# End: ***
\end{verbatim}\end{comment}
} */
