# @(#) makefile for nmake.exe

# ----------------------------------------------------------------------
# makefile
# generate libohymos.a (or ohymos.lib) and make test programs
# ----------------------------------------------------------------------

# macro : package name
PACKAGENAME = ohymos

# macro : user name
USER = ichikawa
# USER = mizukoshi

# macro : ohymos version
OHYMOSVERSION = ohymos-current

# @begin{instructions-for-unix} ----------------------------------------
# OSTYPE = unx
# OHYMOSLIB = libohymos.a
# RELEASEDIR = /tmp
# OHYMOSDIR = /home/$(USER)/ohymos-lib/$(OHYMOSVERSION)
# OHYMOSDIR = /Users/$(USER)/ohymos-lib/$(OHYMOSVERSION)
# 
# HEADERDIR = -I$(OHYMOSDIR)
# LIBDIR    = -L/lib -L$(OHYMOSDIR) -L.
# LIBS      = -lohymos -lm
# 
# USRINCLUDEDIR =
# USRSOURCEDIR  =
# USRLIBDIR     =
# 
# CXXFLAG = -Wall -g $(HEADERDIR)
# CCFLAG = -c -traditional
# LDFLAG = $(LIBDIR) $(LIBS)
# 
# ifndef CXX
# CXX = g++
# endif
# 
# OBJ = .o
# EXE =
# EXN = -o $@
# 
# ifndef CC
# CC = gcc
# endif
# 
# 
# .SUFFIXES:
# .SUFFIXES:	$(OBJ) .cc
# 
# .cc$(OBJ):
# 	$(CXX) -c $*.cc $(CXXFLAG)
# 
# @end{instructions-for-unix} ------------------------------------------

# @begin{instructions-for-gwin} ----------------------------------------
# OSTYPE = gwn
# OHYMOSLIB = libohymos.a
# OHYMOSDIR = /home/$(USER)/ohymos-lib/$(OHYMOSVERSION)
# HEADERDIR = -I$(OHYMOSDIR)
# LIBDIR    = -L/lib -L/usr/lib -L$(OHYMOSDIR) -L.
# LIBS      = -lohymos -lm
# 
# USRINCLUDEDIR =
# USRSOURCEDIR  =
# USRLIBDIR     =
# 
# CXXFLAG = -Wall -g $(HEADERDIR)
# CCFLAG = -c -traditional
# LDFLAG = $(LIBDIR) $(LIBS)
# 
# LD        = g++
# LDFLAGS   = -g -o
# 
# CXX = g++
# OBJ = .o
# EXE = .exe
# EXN = -o $@
# 
# CC = gcc
# 
# .SUFFIXES:
# .SUFFIXES:	$(OBJ) .cc
# 
# .cc$(OBJ):
# 	$(CXX) -c $*.cc $(CXXFLAG)
# 
# @end{instructions-for-gwin} ------------------------------------------

# @begin{instructions-for-vcp} -----------------------------------------
OSTYPE = vcp
OHYMOSLIB = ohymos.lib
OHYMOSDIR = /home/$(USER)/ohymos-lib/$(OHYMOSVERSION)
HEADERDIR = /I:$(OHYMOSDIR)
LIBDIR    = /LIBPATH:$(OHYMOSDIR)
LIBS      = $(OHYMOSLIB)

USRINCLUDEDIR =
USRSOURCEDIR  =
USRLIBDIR     =

LDFLAG = $(LIBS)
OMPFLAG = /Qopenmp
# OMPFLAG = 

CXX = icl
CXXFLAG = /c /GX $(HEADERDIR) $(OMPFLAG)
OBJ = .obj
EXE = .exe
EXN =

.SUFFIXES:
.SUFFIXES:	$(OBJ) .cc .lib

.cc$(OBJ):
	$(CXX) $(CXXFLAG) $*.cc
#	$(CXX) $(CXXFLAG) /Tp $*.cc

# @end{instructions-for-vcp} -------------------------------------------

# ----------------------------------------------------------------------
# Maybe you don't have to edit below this line.
# ----------------------------------------------------------------------

# macro : header files
HDRS = common.h data2str.h datapack.h f_look.h input.h \
element.h input_f.h khmkdir.h list.h newport.h \
object.h ohoption.h \
output.h output_f.h p_tmp.h p_tmp.cc p_itmp.cc port.h \
receive.h \
relay.h scfsub.h scftotal.h send.h sender.h osd_tmp.h sub.h total.h \
vary.h dfopen.h \
portlist.h elmlist.h newelement.h \
ostypeunx.h ostypegwn.h ostypevcp.h

# macro : source files
SRCS = test_e.mak test_e.i0 test_e.ist test_e.pmt \
common.cc data2str.cc \
element.cc elm_t.cc elm_t.h f_look.cc input.cc input_f.cc \
khmkdir.cc list.cc \
newelement.cc newport.cc \
object.cc ohoption.cc osd_tmp.cc output.cc \
output_f.cc \
port.cc receive.cc relay.cc scfsub.cc scftotal.cc send.cc \
sender.cc sub.cc test_e.cc total.cc vary.cc \
dfopen.cc

# macro : object files
OBJS = common$(OBJ) data2str$(OBJ) element$(OBJ) \
f_look$(OBJ) input$(OBJ) input_f$(OBJ) \
khmkdir$(OBJ) list$(OBJ) object$(OBJ) ohoption$(OBJ) \
output$(OBJ) output_f$(OBJ) port$(OBJ) p_itmp$(OBJ) \
receive$(OBJ) relay$(OBJ) scfsub$(OBJ) scftotal$(OBJ) \
send$(OBJ) sender$(OBJ) osd_tmp$(OBJ) \
sub$(OBJ) total$(OBJ) \
vary$(OBJ) dfopen$(OBJ)

# macro : documents
AUXS = 00Readme.e 00Readme.j 01Changes.j 02Install.j \
	version.e version.cc \
	Makefile Makefile.icl Makefile.icpc Makefile.gomp Makefile.vcp

DATA =

AUXBIN = toeucdos tosjis

# macro : test programs
TESTPROG = test_e$(EXE)

# ----------------------------------------------------------------------
# rules
# ----------------------------------------------------------------------

all	:	$(OHYMOSLIB) ostype.h test 

ostype.h	:
	cp ostype$(OSTYPE).h ostype.h

#DFILES  = $(OBJS:$(OBJ)=.d)

#%.d : %.cc
#	$(CXX) -M $< > $@

#depend	: $(DFILES)

#include $(DFILES)


# ----------------------------------------------------------------------
# generates libohymos.a or ohymos.lib
# ----------------------------------------------------------------------

# compile : basic tools
common$(OBJ)	:	common.cc common.h

data2str$(OBJ)	:	data2str.cc data2str.h

dfopen$(OBJ)	:	dfopen.cc dfopen.h

list$(OBJ)		:	list.cc list.h

f_look$(OBJ)	:	f_look.cc f_look.h

object$(OBJ)	:	object.cc object.h

ohoption$(OBJ)	:	ohoption.cc ohoption.h

osd_tmp$(OBJ)	:	osd_tmp.cc osd_tmp.h

element$(OBJ)	:	element.cc element.h object$(OBJ)

scfsub$(OBJ)		:	scfsub.cc scfsub.h element$(OBJ)

scftotal$(OBJ)		:	scftotal.cc scftotal.h element$(OBJ)

sub$(OBJ)		:	sub.cc sub.h element$(OBJ)

total$(OBJ)		:	total.cc total.h sub$(OBJ) element$(OBJ)

port$(OBJ)		:	port.cc port.h object$(OBJ)

receive$(OBJ)	:	receive.cc receive.h port$(OBJ)

sender$(OBJ)	:	sender.cc sender.h port$(OBJ)

send$(OBJ)		:	send.cc send.h sender$(OBJ)

relay$(OBJ)		:	relay.cc relay.h sender$(OBJ)

input$(OBJ)		:	input.cc input.h send$(OBJ)

output$(OBJ)	:	output.cc output.h receive$(OBJ)

input_f$(OBJ)	:	input_f.cc input_f.h input$(OBJ)

output_f$(OBJ)	:	output_f.cc output_f.h output$(OBJ)

vary$(OBJ)		:	vary.cc vary.h data2str.h

# compile : template
p_itmp$(OBJ)	:	p_itmp.cc p_tmp.cc p_tmp.h \
	datapack.h port$(OBJ) sender$(OBJ) send$(OBJ) \
        receive$(OBJ) relay$(OBJ) input$(OBJ) input_f$(OBJ) \
	output$(OBJ) output_f$(OBJ)

# compile : io_file_plus
#io_file_plus$(OBJ)	: io_file_plus.cc io_file_plus.h \
#	p_itmp$(OBJ) data2str.h

# link : libohymos.a

# @begin{instructions-for-unix} ----------------------------------------
# libohymos.a		:	$(OBJS)
# 	ar rcv libohymos.a $? ; ranlib $@
# 
# install	:
# 	/usr/local/bin/install -m 644 $(HDRS) $(USRINCLUDEDIR)
# 	/usr/local/bin/install -m 644 libohymos.a $(USRLIBDIR)
# 	/usr/local/bin/install -m 644 $(HDRS) $(SRCS) $(DOCS) \
# 	  $(USRSOURCEDIR)
# 	cd $(USRLIBDIR); ranlib libohymos.a;
# @end{instructions-for-unix} ------------------------------------------

# @begin{instructions-for-gwin} ----------------------------------------
# libohymos.a		:	$(OBJS)
# 	ar rcv libohymos.a $? ; ranlib $@
# 
# install	:
# 	/usr/local/bin/install -m 644 $(HDRS) $(USRINCLUDEDIR)
# 	/usr/local/bin/install -m 644 libohymos.a $(USRLIBDIR)
# 	/usr/local/bin/install -m 644 $(HDRS) $(SRCS) $(DOCS) \
# 	  $(USRSOURCEDIR)
# 	cd $(USRLIBDIR); ranlib libohymos.a;
# @end{instructions-for-gwin} ------------------------------------------

# @begin{instructions-for-vcp} -----------------------------------------
ohymos.lib		:	$(OBJS)
	lib /out:ohymos.lib $(OBJS)

install	:
	copy $(HDRS) $(USRINCLUDEDIR)
	copy ohymos.lib $(USRINCLUDEDIR)
	copy $(HDRS) $(SRCS) $(DOCS) $(USRSOURCEDIR)
# @end{instructions-for-vcp} -------------------------------------------

# target to install (1) header files (2) source files (3) libohymos.a

# ----------------------------------------------------------------------
# make a test program
# ----------------------------------------------------------------------

test	:	$(TESTPROG)

test_e$(EXE)	:	test_e$(OBJ) elm_t$(OBJ) $(OHYMOSLIB)
	$(CXX) $(EXN) test_e$(OBJ) elm_t$(OBJ) $(LDFLAG)

test_e$(OBJ)	:	test_e.cc

elm_t$(OBJ)	:	elm_t.cc elm_t.h

clear	:	clean

clean	:
	-rm -f core *.core a$(EXE) *$(OBJ) *~ $(OHYMOSLIB)
	-rm -f *-cc.* *-h.* ostype.h $(DFILES)
	-rm -f $(TESTPROG)
	cd parts; $(MAKE) clean

eucdos	:
	toeucdos $(SRCS) $(AUXS) $(DATA)

sjis	:
	tosjis $(SRCS) $(AUXS) $(DATA)

