/*;  vary.h {\documentstyle[fleqn]{jsrc2latex}\font\tenrm=cmtt10} */

/*;  $$Revision: 1.1 $$ {\rhead{\sl vary.h\quad\thepage}} */
/*;  $$Date: 2000/12/02 08:55:20 $$  */
/*;  $$Source: /home/shiiba/ohymos-lib/ohymos-current/vary.h,v $$  */
/*;  $$RCShost: yodo.kuciv.kyoto-u.ac.jp $$  */

// @(#) vary.h -- template class for treating an array

/*
Class Vary<T> is a template class for treating an array of type T. The
number of elements is also contained as its data member.

For example, the following line declares an array of type double with
the size of 3,

      Vary<double> a(3);
      int n = a.Size(); // n = 3
      a[0] = 1.2;
*/

#if !defined(_vary_h)
#define _vary_h

#include <stdio.h>

#include "data2str.h" // added by Y.Ichikawa (Thu Feb 15 15:50:32 1996)

/*----------------------------------------------------------------------
CLASS
    Vary

    Template class for treating a variable size array

DESCRIPTION

    Class Vary&lt;T&gt; is a template class for treating an array of type
    T. The number of elements is also contained as its data member.

    For example, the following line declares an array of type double
    with the size of 3,

    EXAMPLE
      Vary<double> a(3);
      int n = a.Size(); // n = 3
      a[0] = 1.2;
    END

----------------------------------------------------------------------*/

template <class Type>
class Vary {
  Type* v;
  int size;

public:
  //////////////////////////////////////////////////////////////////////
  // Changes the size to n
  void Resize(int n);

  //////////////////////////////////////////////////////////////////////
  // Returns the i-th component
  Type Value(int i) const { return v[i]; }

  //////////////////////////////////////////////////////////////////////
  // return the size of the array
  int Size(void) const { return size; }

  //////////////////////////////////////////////////////////////////////
  // constructor
  Vary(void);

  //////////////////////////////////////////////////////////////////////
  // constructor
  Vary(int n);

  //////////////////////////////////////////////////////////////////////
  // copy constructor
  Vary(const Vary<Type>& a);

  //////////////////////////////////////////////////////////////////////
  // substitution constructor
  Vary<Type>& operator=(const Vary<Type>& a);

  //////////////////////////////////////////////////////////////////////
  // destructor
  ~Vary();

  //////////////////////////////////////////////////////////////////////
  // you can use a[i] as a[i] = 3.
  Type& operator[](int idx) { return v[idx]; }
};

template <class Type>
void dstr2data(char*& dstr, Vary<Type>& a);

template <class Type>
void data2file(Vary<Type>& a, FILE* fp);

template <class Type>
void data2ofile(Vary<Type>& a, FILE* fo);

#endif // ! _vary_h

/*; Local Variables: *** */
/*; tab-width: 4 *** */
/*; fill-column: 72 *** */
/*; End: *** */

/*; end of file */
